const conn = require("../config/database");
const GLOBALS = require("../config/constant");
const asyncLoop = require("node-async-loop");
const { t } = require("localizify");
const nodemailer = require("nodemailer");
const otpGenerator = require("otp-generator");
const { default: localizify } = require("localizify");
const middleware = require("../middleware/validation");
const moment = require("moment");

// todo: Please change the function or other stuff according to your needs.

const common = {
    // ! This function is used to send email via Nodemailer
    sendEmail: function (to_email, subject, message, callback) {
        const transporter = nodemailer.createTransport({
            host: "smtp.gmail.com",
            auth: {
                user: process.env.EMAIL_ID,
                pass: process.env.EMAIL_PASSWORD,
            },
        });

        const mailOptions = {
            from: process.env.EMAIL_ID,
            to: to_email,
            subject: subject,
            html: message,
        };

        transporter.sendMail(mailOptions, function (error, info) {
            if (error) {
                callback(false);
            } else {
                callback(true);
            }
        });
    },

    // ! For encrypt the response data
    encryption: function (response_data, callback) {
        const response = cryptLib.encrypt(JSON.stringify(response_data), shaKey, process.env.IV);
        callback(response);
    },

    //  ! For sending response of the API
    send_response: function (req, res, code, message, data) {
        common.getMessage(req.language, message, (translated_message) => {
            responseData = { code: code, message: translated_message, data: data };

            // * This function is used to send encrypted response

            middleware.encryption(responseData, (encrypted_message) => {
                res.status(200);
                res.send(encrypted_message);
            });

            // if (responseData.code == 9) {
            //     res.status(404).send(responseData);
            // } else {
            //     res.status(200).send(responseData);
            // }
        });
    },

    // ! For getting the translated message
    getMessage: (language, message, callback) => {
        // localizify.add('en',en).add('gu',gu).add('hi',hi).setLocale(language);

        callback(t(message.keyword, message.content));
    },

    // ! This function is used for getting the user details
    getUserDetails: (userId, callback) => {
        conn.query("SELECT * FROM tbl_user WHERE id = ? AND is_delete = 0", [userId], (error, result) => {
            if (error) {
                callback([]);
            } else {
                if (result.length > 0) {
                    // * To get user location or user device details if there are multiple

                    // asyncLoop(result, (items, next) => {
                    //     conn.query('SELECT * FROM tbl_user_device AS d WHERE d.user_id = ?', [userId], (error, results) => {
                    //         if (!error) { items.device_info = results; }
                    //     });
                    //     conn.query('SELECT * FROM tbl_user_location AS l WHERE l.user_id = ?', [userId], (error, results) => {
                    //         if (!error) { items.location_info = results; next(); }
                    //     });
                    // common.getDeviceInfo(items.id,(results)=>{
                    //     items.device_info = results; next();
                    // })
                    // }, (error) => { if (!error) { callback(result); } else{ callback([]); } });

                    callback(result);
                } else {
                    callback([]);
                }
            }
        });
    },

    // ! For checking the dob is valid or not
    dobIsValid: (dob) => {
        // ! For the  Format : YYYY-MM-DD
        const rex = /^\d{4}-\d{2}-\d{2}$/;

        // ! For the  Format : DD-MM-YYYY
        // const rex = /^\d{2}-\d{2}-\d{4}$/;

        return rex.test(dob);
    },

    // ! This function is used to get password of particular user
    getPassword: (userId, callback) => {
        conn.query("SELECT u.password FROM tbl_user AS u WHERE u.id = ? AND u.is_active = 1 AND u.is_delete = 0", [userId], (error, result) => {
            if (!error && result.length > 0) {
                callback(result[0]);
            }
        });
    },

    // ! This function is used to insert the device info . It will also check whether device info is there or not.
    // ! If there then it will be updated or else inserted.
    insertUserDeviceInfo: (deviceData, callback) => {
        conn.query("SELECT * FROM tbl_user_device AS u WHERE u.is_active = 1 AND u.is_delete = 0 AND u.user_id = ? ", [deviceData.user_id], (error, result) => {
            if (error) {
                callback(false);
            } else {
                if (result.length > 0) {
                    conn.query("UPDATE tbl_user_device AS d SET ? WHERE d.id = ?", [deviceData, result[0].id], (error, result) => {
                        if (error) {
                            callback(false);
                        } else {
                            if (result.affectedRows > 0) {
                                callback(true);
                            }
                        }
                    });
                } else {
                    conn.query("INSERT INTO tbl_user_device SET ?", [deviceData], (error, result) => {
                        if (error) {
                            callback(false);
                        } else {
                            if (result.affectedRows > 0) {
                                callback(true);
                            }
                        }
                    });
                }
            }
        });
    },

    // ! This function is used to insert the location info . It will also check whether location info is there or not.
    // ! If there then it will be updated or else inserted.
    insertLocationInfo: (deviceData, callback) => {
        conn.query("SELECT * FROM tbl_user_location AS l WHERE l.is_active = 1 AND l.is_delete = 0 AND l.user_id = ? ", [deviceData.user_id], (error, result) => {
            if (error) {
                callback(false);
            } else {
                if (result.length > 0) {
                    conn.query("UPDATE tbl_user_location AS l SET ? WHERE l.id = ?", [deviceData, result[0].id], (error, result) => {
                        if (error) {
                            callback(false);
                        } else {
                            if (result.affectedRows > 0) {
                                callback(true);
                            }
                        }
                    });
                } else {
                    conn.query("INSERT INTO tbl_user_location SET ?", [deviceData], (error, result) => {
                        if (error) {
                            callback(false);
                        } else {
                            if (result.affectedRows > 0) {
                                callback(true);
                            }
                        }
                    });
                }
            }
        });
    },

    // ! This function is used to generate the random token. Or else you can use Otp Genrator node package manager.
    generateToken: function () {
        const alphnumeric = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789";
        let randomCode = "";

        for (let i = 0; i < 5; i++) {
            const randomIndex = Math.floor(Math.random() * alphnumeric.length);
            randomCode += alphnumeric[randomIndex];
        }

        return randomCode;
    },

    //  //! This function is used to generate the random otp. Or else you can use Otp Genrator node package manager.
    generateOTP: function (length) {
        return Math.floor(1000 + Math.random() * 9000);
    },

    // ! This function is used to get device info of particular user.
    getDeviceInfo: (userId, callback) => {
        conn.query("SELECT * FROM tbl_user_device AS d WHERE d.user_id = ?", [userId], (error, result) => {
            if (error) {
                callback(error);
            } else {
                if (result.length > 0) {
                    callback(result);
                } else {
                    callback(["Result Not Found"]);
                }
            }
        });
    },

    // ! For checking the email format
    checkEmail: (email) => {
        const mailformat = /^(([^<>()[\]\.,;:\s@\"]+(\.[^<>()[\]\.,;:\s@\"]+)*)|(\".+\"))@(([^<>()[\]\.,;:\s@\"]+\.)+[^<>()[\]\.,;:\s@\"]{2,})$/i;

        return mailformat.test(email);
    },

    // ! For checking the phone format ( Valid 10 digits )
    checkPhone: (phone) => {
        const phoneformat = /[0-9]{10}/g;

        return phoneformat.test(phone);
    },

    // ! This function is used to check if email already exists in the databse or not
    uniqueEmail: async function (tableName, email, callback) {
        try {
            if (email != undefined && email != "") {
                const check_email = `SELECT id FROM ${tableName} WHERE email = '${email}' AND is_active = true AND is_delete = false;`;

                const data = await conn.query(check_email);
                if (data.rows.length == 0) {
                    return true;
                } else {
                    return false;
                }
            } else {
                callback(true);
            }
        } catch (error) {
            throw error;
        }
    },

    // ! This function is used to check if mobile number already exists in the databse or not
    uniqueMobile: function (mobile, callback) {
        if (mobile != undefined) {
            const check_mobile = `SELECT id FROM tbl_user WHERE mobile_number = ? AND country_code = ? AND role = ? AND is_active = 1 AND 
            is_delete = 0;`;

            conn.query(check_mobile, [mobile, country_code, role], function (error, mobile_check) {
                if (!error && mobile_check.length == 0) {
                    return callback(true, null);
                } else {
                    return callback(false, mobile_check[0]);
                }
            });
        } else {
            return callback(true, null);
        }
    },

    // ! This function is used to check if mobile number with country code already exists in the databse or not
    uniqueMobileCountryCode: async function (mobile, countryCode, tableName) {
        try {
            if (mobile != undefined || countryCode != undefined) {
                const check_mobile = `SELECT id FROM ${tableName} WHERE mobile_number = ${mobile} AND country_code = '${countryCode}' AND is_active = true AND is_delete = false;`;

                const data = await conn.query(check_mobile);
                if (data.rows.length == 0) {
                    return true;
                } else {
                    return false;
                }
            } else {
                return true;
            }
        } catch (error) {
            throw error;
        }
    },

    // ! This function is used to check if username already exists in the databse or not
    uniqueUsername: function (username, callback) {
        const check_username = `SELECT id FROM tbl_user WHERE username = ? AND is_active = 1 AND is_delete = 0;`;

        conn.query(check_username, [username], function (error, username_check) {
            if (!error && username_check.length == 0) {
                callback(true);
            } else {
                callback(false);
            }
        });
    },

    // ! This function is used to check if social id already exists in the databse or not
    uniqueSocialID: function (socialId, callback) {
        const check_social = `SELECT id FROM tbl_user WHERE social_id = ? AND is_active = 1 AND is_delete = 0;`;
        const socialID = [socialId];

        conn.query(check_social, socialID, function (error, social_check) {
            if (!error && social_check.length == 0) {
                callback(true, null);
            } else {
                callback(false, social_check[0]);
            }
        });
    },

    // ! This function is used get particular product details.
    getProductDetails: (product_id, callback) => {
        conn.query(`SELECT * FROM tbl_dishes WHERE is_active = 1 AND id = ?`, [product_id], (error, result) => {
            if (error) {
                return callback([]);
            } else {
                if (result.length > 0) {
                    return callback(result);
                } else {
                    return callback([]);
                }
            }
        });
    },

    // ! This function basically checks whe+ther the product is exist in cart details or not.
    // ! If it doesn't exist it will insert it else it will update the quantity.
    // ! Please analysis the flow and then do changes according if needed
    insertCartDetails: async (cartData) => {
        try {
            const cartDetails = await common.getCommonMultipleRecord("customer.tbl_cart_details", `cart_id = ${cartData.cart_id} AND product_id = ${cartData.product_id}`);

            if (cartDetails != null) {
                cartData.quantity = parseInt(cartData.quantity);

                const result = await common.singleUpdate(
                    "customer.tbl_cart_details",
                    { quantity: cartData.quantity, total: cartData.total },
                    `cart_id =  ${cartData.cart_id} AND product_id = ${cartData.product_id}`
                );

                if (result > 0) {
                    return result;
                } else {
                    return null;
                }
            } else {
                const result = await common.singleInsert("customer.tbl_cart_details", cartData);

                if (result > 0) {
                    return result;
                } else {
                    return null;
                }
            }
        } catch (error) {
            throw error;
        }
    },

    // ! For getting cart details from cart id.
    getCartDetails: (cartId, callback) => {
        conn.query(`SELECT * FROM tbl_cart_details WHERE is_active = 1 AND cart_id = ?`, [cartId], (error, result) => {
            if (error) {
                return callback(null);
            } else {
                if (result.length > 0) {
                    return callback(result);
                } else {
                    return callback(null);
                }
            }
        });
    },

    // ! To insert order details in the table order-details.
    insertOrderDetails: (orderData, callback) => {
        conn.query(`INSERT INTO tbl_order_details SET ?`, [orderData], (error, result) => {
            if (error) {
                return callback(false);
            } else {
                if (result.insertId > 0) {
                    return callback(true);
                } else {
                    return callback(false);
                }
            }
        });
    },

    // ! For deleting cart details from both tbl-cart and tbl-cart-details using cart-id.
    deleteCartDetails: (cartId, callback) => {
        conn.query(`UPDATE tbl_cart_details SET is_active = 0 WHERE is_active = 1 AND cart_id = ?`, [cartId], (error, result) => {
            if (error) {
                return callback(false);
            } else {
                if (result.affectedRows > 0) {
                    conn.query(`UPDATE tbl_cart SET is_active = 0 , is_delete = 1 WHERE is_active = 1 AND is_delete = 0 AND id = ?`, [cartId], (error, result) => {
                        if (error) {
                            return callback(false);
                        } else {
                            if (result.affectedRows > 0) {
                                return callback(true);
                            } else {
                                return callback(false);
                            }
                        }
                    });
                } else {
                    return callback(false);
                }
            }
        });
    },

    // ! For inserting the notifications in notification table.
    insertNotification: function (notification, callback) {
        conn.query("INSERT INTO tbl_notifications SET ?", notification, function (err, result, fields) {
            if (!err && result.insertId > 0) {
                callback(true);
            } else {
                callback(false);
            }
        });
    },

    // ! For inserting the card details in tbl-card table.
    insertCardDetails: function (cardData, callback) {
        conn.query("INSERT INTO tbl_card_details SET ?", [cardData], function (err, result, fields) {
            if (!err && result.insertId > 0) {
                callback(true);
            } else {
                callback(false);
            }
        });
    },

    // ! For updating the order status of particular order using order-id in tbl-order table.
    updateOrderStatus: (orderId, callback) => {
        conn.query("UPDATE tbl_order AS o SET o.status = 'confirmed' WHERE  o.id = ?", [orderId], function (err, result) {
            if (!err && result.affectedRows > 0) {
                callback(true);
            } else {
                callback(false);
            }
        });
    },

    // ! This function is used to insert payment data. Please analyse it and change the flow as required.
    paymentGateway: (requestData, callback) => {
        // let paymentObj = {
        //     order_id: requestData.order_id,
        //     user_id: requestData.user_id,
        //     payment_mode_id: requestData.payment_mode_id,
        //     transaction_id: 1234,
        // };

        conn.query(`INSERT INTO tbl_payment SET ?`, [requestData], (error, result) => {
            if (error) {
                return callback(false);
            } else {
                if (result.insertId > 0) {
                    if (requestData.payment_mode_id == 2) {
                        // let carDetails = {
                        //     user_id: requestData.user_id,
                        //     card_holder_name: requestData.card_holder_name,
                        //     card_number: requestData.card_number,
                        //     country_code: requestData.country_code,
                        //     zip_code: requestData.zip_code,
                        //     expiry_month: requestData.expiry_month,
                        //     expiry_date: requestData.expiry_date,
                        // };

                        // conn.query(`INSERT INTO tbl_card_details SET ?`, [carDetails], (error, nestedresult) => {
                        //     if (error) {
                        //         return callback(false);
                        //     } else {
                        //         if (nestedresult.insertId > 0) {
                        //             return callback(true);
                        //         } else {
                        //             return callback(false);
                        //         }
                        //     }
                        // });

                        return callback(true);
                    } else {
                        return callback(true);
                    }
                } else {
                    return callback(false);
                }
            }
        });
    },

    // ! This function is used to get multiplerecord . Please analyse it and change the flow as required, you have to pass the table name and conditonal data as per the requirement.

    getCommonMultipleRecord: async function (tablename, condition) {
        const query = `SELECT * , TO_CHAR(created_at, 'YYYY-MM-DD HH24:MI:SS') AS created_at , TO_CHAR(updated_at, 'YYYY-MM-DD HH24:MI:SS') AS updated_at FROM ${tablename} WHERE ${condition} ORDER BY id ASC`;

        try {
            const result = await conn.query(query, []);

            if (result.rows.length > 0) {
                return result.rows;
            } else {
                return null;
            }
        } catch (error) {
            throw error;
        }

        // conn.query(query, [], (error, result) => {
        //     if (!error) {
        //         if (result.rows.length > 0) {
        //             return result;
        //         } else {
        //             return null;
        //         }
        //     } else {
        //         throw error;
        //     }
        // });
    },

    // ! This funmction is used to get only single record from the database. Please analyse it and change as per the flow as required, you have to pass the table name and conditional data as per the requirement   ex------> `id=${id}`,  `email='${email}'`
    getCommonSingleRecord: async function (tablename, condition) {
        const query = `SELECT * , TO_CHAR(created_at, 'YYYY-MM-DD HH24:MI:SS') AS created_at , TO_CHAR(updated_at, 'YYYY-MM-DD HH24:MI:SS') AS updated_at FROM ${tablename} WHERE ${condition}`;

        try {
            const result = await conn.query(query, []);

            if (result.rows.length > 0) {
                return result.rows[0];
            } else {
                return null;
            } // Get the first record
        } catch (error) {
            throw error;
        }
    },

    // ! This function is used to insert single data in any table

    // ! This is the way how the data to be passed in the below functions. Please analyse it and change according if required.

    // let query = `INSERT INTO tbl_user (first_name, last_name, email, password) VALUES ($1, $2, $3, $4) RETURNING *`;
    // let values = [
    //     request.first_name,
    //     request.last_name,
    //     request.email,
    //     request.password
    // ];

    // conn.query(query,values).then((response)=>{

    //         callback(
    //             "1",
    //             {
    //               keyword: "rest_keyword_sucess",
    //               content: "",
    //             },
    //             []
    //           );
    // }).catch((error)=>{
    //      callback("0",
    //     {
    //       keyword: "rest_keyword_error",
    //       content: "",
    //     },
    //     error.sqlmessage    signup:async function(request,callback){
    // let query = `INSERT INTO tbl_user (first_name, last_name, email, password) VALUES ($1, $2, $3, $4) RETURNING *`;
    // let values = [
    //     request.first_name,
    //     request.last_name,
    //     request.email,
    //     request.password
    // ];
    //   );
    //  })

    //! This function is used to insert data for single row.

    singleInsert: async function (tablename, params) {
        params.created_at = moment.utc().format("YYYY-MM-DD HH:mm:ss");
        params.updated_at = moment.utc().format("YYYY-MM-DD HH:mm:ss");

        const columns = Object.keys(params).join(", ");
        const values = Object.values(params);
        const placeholders = values.map((_, index) => `$${index + 1}`).join(", ");
        const query = `INSERT INTO ${tablename} (${columns}) VALUES (${placeholders}) RETURNING id`;

        try {
            const result = await conn.query(query, values);

            if (result.rows.length > 0) {
                return result.rows[0].id;
            } else {
                return null;
            }
        } catch (error) {
            throw error;
        }
    },

    // ! This funmction is used to update only single record from the database. Please analyse it and change as per the flow as required, you have to pass the table name and conditional data as per the requirement   ex------> `id=${id}`,  `email='${email}'`

    singleUpdate: async function (tablename, params, condition) {
        params.updated_at = moment.utc().format("YYYY-MM-DD HH:mm:ss");

        const columns = Object.keys(params)
            .map((key, index) => `${key} = $${index + 1}`)
            .join(", ");
        const values = Object.values(params);
        const query = `UPDATE ${tablename} SET ${columns} WHERE ${condition} RETURNING id;`;
        try {
            const result = await conn.query(query, values);

            return result.rows[0].id;
        } catch (error) {
            throw error;
        }
    },

    // ! This funmction is used to get only single record from the database. Please analyse it and change as per the flow as required, you have to pass the table name and conditional data as per the requirement   ex------> `id=${id}`,  `email='${email}'`

    singleSelect: function (tablename, condition, callback) {
        const query = `SELECT * FROM ${tablename} WHERE ${condition}`;
        conn.query(query, [], (error, result) => {
            if (!error) {
                callback(null, result.rows[0]); // Return the first record
            } else {
                callback(error, null);
            }
        });
    },

    // ! This funmction is used to delete only single record from the database. Please analyse it and change as per the flow as required, you have to pass the table name and conditional data as per the requirement   ex------> `id=${id}`,  `email='${email}'`
    singleDelete: async function (tablename, condition, callback) {
        const query = `DELETE FROM ${tablename} WHERE ${condition} AND is_active = true AND is_delete = false RETURNING id;`;

        const result = await conn.query(query, []);

        if (result.rows.length == 0) {
            return null;
        } else {
            return result.rows[0].id;
        }
    },

    // ! This funmction is used to get userModel by using particular user id for getting user details which includes all user details with deviceinfo of that user.
    getCustomerModel: async function (condition) {
        const query = `SELECT * , TO_CHAR(date_of_birth, 'YYYY-MM-DD') AS date_of_birth , TO_CHAR(created_at, 'YYYY-MM-DD HH24:MI:SS') AS created_at , TO_CHAR(updated_at, 'YYYY-MM-DD HH24:MI:SS') AS updated_at FROM customer.tbl_customer WHERE ${condition} AND is_active = true AND is_delete = false`;

        try {
            const result = await conn.query(query, []);

            if (result.rows.length == 0) {
                return null; // No customer found
            }

            const data = {
                customer_details: result.rows[0],
            };

            // Fetch associated device data
            const customer_device_data = await common.getCommonSingleRecord("customer.tbl_customer_device", `customer_id = ${result.rows[0].id}`);

            if (customer_device_data != null) {
                data.device_info = customer_device_data;
            } else {
                data.device_info = [];
            }

            const customer_location_data = await common.getCommonSingleRecord("customer.tbl_location_history", `customer_id = ${result.rows[0].id} `);

            if (customer_location_data != null) {
                data.location_data = customer_location_data;
            } else {
                customer_location_data = [];
            }

            return data; // Return the collected data
        } catch (error) {
            throw error; // Propagate the error to the caller
        }
    },

    // ! This funmction is used to get userModel by using particular user id for getting user details which includes all user details with deviceinfo of that user.
    getDriverModel: async function (condition) {
        const query = `SELECT * , TO_CHAR(date_of_birth, 'YYYY-MM-DD') AS date_of_birth , TO_CHAR(created_at, 'YYYY-MM-DD HH24:MI:SS') AS created_at , TO_CHAR(updated_at, 'YYYY-MM-DD HH24:MI:SS') AS updated_at FROM driver.tbl_driver WHERE ${condition} AND is_active = true AND is_delete = false`;

        try {
            const result = await conn.query(query, []);

            if (result.rows.length == 0) {
                return null; // No customer found
            }

            const data = {
                driver_details: result.rows[0],
            };

            // Fetch associated device data
            const driver_device_data = await common.getCommonSingleRecord("driver.tbl_driver_device", `driver_id = ${result.rows[0].id}`);

            if (driver_device_data != null) {
                data.device_info = driver_device_data;
            } else {
                data.device_info = [];
            }

            return data; // Return the collected data
        } catch (error) {
            throw error; // Propagate the error to the caller
        }
    },
};

module.exports = common;
