const common = require("../config/common");
require("dotenv").config();
const stripe = require("stripe")(process.env.STRIPE_AUTH_API_KEY);

let stripe_modules = {
    // * to create user in stripe account

    createUser: (userId, callback) => {
        common.getUserDetails(userId, async (response) => {
            const customer = await stripe.customers.create({
                name: response[0].username,
                email: response[0].email,
            });
            callback(customer);
        });
    },

    // * to get user details from stripe dahboard using email

    getUser: async (email, callback) => {
        const customers = await stripe.customers.list({
            email: email,
        });

        callback(customers);
    },

    // * to create the payment page using checkout session or payment link method

    createPaymentLink: async (lineItems, callback) => {
        // ! For creating a single price payment link

        // const paymentLink = await stripe.paymentLinks.create({
        //     line_items: lineItems,
        // });

        // ! For showing multiple products data in payement page use below lineItem map functioon

        // const lineItems = result.map((element) => ({
        //     price_data: {
        //         currency: "inr",
        //         product_data: {
        //             name: element.product_name,
        //             images: [element.image],
        //         },
        //         unit_amount: element.per_dish_price * 100,
        //     },
        //     quantity: element.total_qty,
        // }));

        const session = await stripe.checkout.sessions.create({
            success_url: "http://localhost:4200/payment-success",
            cancel_url: "http://localhost:4200/payment-failed",
            line_items: lineItems,
            mode: "payment",
        });

        callback(session);
    },

    // * to create a subscription

    createSubscription: async (price_id, customer_id, callback) => {
        // ! For making the subscriptions payment page

        const subscription = await stripe.subscriptions.create({
            customer: customer_id,
            items: [
                {
                    price: price_id,
                },
            ],
        });

        //        // console.log(subscription);

        // callback(subscription);

        // const invoice = await stripe.invoices.create({
        //     customer: customer_id,
        //     subscription: subscription.id,
        //     collection_method: "send_invoice",
        //     days_until_due: "0",
        // });

        //        // // console.log(invoice);

        // callback(invoice);

        // const session = await stripe.checkout.sessions.create({
        //     customer: customer_id,
        //     success_url: "http://localhost:4200/payment-success",
        //     cancel_url: "http://localhost:4200/payment-failed",
        //     line_items: [
        //         {
        //             price: price_id,
        //             quantity: 1,
        //         },
        //     ],
        //     mode: "subscription",
        // });

        // callback(session);
    },

    // * To check whether user susbscribed or not

    getSubscriptionList: async (customer_id, price_id, callback) => {
        const subscriptions = await stripe.subscriptions.list({
            customer: customer_id,
            price: price_id,
        });

        callback(subscriptions);
    },

    // * To create a invoice
    createInvoice: async (subscription_id, customer_id, callback) => {
        const invoice = await stripe.invoices.create({
            customer: customer_id,
            subscription: subscription_id,
        });

        callback(invoice);
    },
};

module.exports = stripe_modules;
