// ? This node package Validator is used to check rules and regulation while getting data(request.body) from the API if any...
const Validator = require("Validator");

// ? To get common file functions
const common = require("../config/common");

// ? To run any query in the database connction is used.
const conn = require("../config/database");

// ? This bypassMethod is basically used to exclude from token validations. Please mention API endpoints here.
const bypassMethods = new Array("login", "change-password", "forgot-password-mobile-verification", "otp-verification", "request-otp", "sign-up");

// ? This bypassMethod is basically used to exclude from token validations. Please mention API endpoints here.
const bypassDriverMethods = new Array("login", "change-password", "forgot-password-mobile-verification", "otp-verification", "request-otp", "sign-up");

// ? This is used to get localizify node package in order to get data or transfer the data or response in specified language.
const { default: localizify } = require("localizify");

// ?  This is used to access the localizify with the help of variable.
const { t } = require("localizify");

// ? This file is used to get response messages of particular language.
const en = require("../language/en");
const gu = require("../language/gu");

// ? To get request body validations.
const lodash = require("lodash");

// ? It is used to encrypt data through cryptlib node package manager. You can also use  other packages like cryptoJS.
const cryptLib = require("cryptlib");

// ? It is used to generate a secret key which will be used to encrypt and decrypt the data using cryptoLib
const shaKey = cryptLib.getHashSha256(process.env.KEY, 32);

const middleware = {
    // ! For checking required validation rules
    checkValidationRules: (request, res, rules, message) => {
        const v = Validator.make(request, rules, message);

        if (v.fails()) {
            let error = "";

            const errors = v.getErrors();

            for (let i in errors) {
                error = errors[i][0];
                break;
            }

            response_data = { code: "0", message: error };

            // * To function is used to encrypt the response data

            middleware.encryption(response_data, (response) => {
                res.status(200).send(response);
            });

            // res.status(401).send(response_data);

            return false;
        } else {
            return true;
        }
    },

    // ! For checking the validation of the api key in the headers
    validateApiKey: function (req, res, callback) {
        const api_key = req.headers["api_key"] != undefined && req.headers["api_key"] != "" ? req.headers["api_key"] : "";

        if (api_key != "") {
            try {
                const apiKey = cryptLib.decrypt(api_key, shaKey, process.env.IV);

                if (apiKey != "" && apiKey == process.env.API_KEY) {
                    callback();
                } else {
                    response_data = {
                        code: 0,
                        message: t("rest_keyword_invalid_api_key"),
                    };

                    // * This function is used to encrypt the response data

                    middleware.encryption(response_data, (response) => {
                        res.status(401);
                        res.send(response);
                    });

                    // res.status(401).send(response_data);
                }
            } catch (error) {
                response_data = {
                    code: 0,
                    message: t("rest_keyword_invalid_api_key"),
                };

                // * This function is used to encrypt the response data

                middleware.encryption(response_data, (response) => {
                    res.status(401);
                    res.send(response);
                });

                // res.status(401).send(response_data);
            }
        } else {
            response_data = {
                code: 0,
                message: t("rest_keyword_invalid_api_key"),
            };

            // * This function is used to encrypt the response data

            middleware.encryption(response_data, (response) => {
                res.status(401);
                res.send(response);
            });

            // res.status(401).send(response_data);
        }
    },

    // ! For checking the validation of the token to get user id
    validateCustomerToken: async function (req, res, callback) {
        const headertoken = req.headers["token"] != undefined && req.headers["token"] != "" ? req.headers["token"] : "";

        const path = req.path.split("/");

        if (bypassMethods.indexOf(path[1]) === -1) {
            if (headertoken != "") {
                try {
                    const token = cryptLib.decrypt(headertoken, shaKey, process.env.IV);

                    // ! If you want to use jwtToken to create token.
                    // const decode = jwtToken.verify(result[0].token, process.env.KEY);

                    try {
                        // const result = await common.getCommonSingleRecord("customer.tbl_customer_device", `token = '${headertoken}'`);

                        const query = `SELECT * FROM customer.tbl_customer_device  WHERE token = '${token}' AND is_active = true AND is_delete = false; `;

                        const result = await conn.query(query, []);

                        if (result.rows.length > 0) {
                            if (req.body === undefined) {
                                req.body = {};
                                req.body.customer_id = result.rows[0].customer_id;
                                callback();
                            } else {
                                req.body.customer_id = result.rows[0].customer_id;

                                callback();
                            }
                        } else {
                            response_data = {
                                code: 0,
                                message: t("rest_keyword_invalid_token"),
                            };

                            middleware.encryption(response_data, (response) => {
                                res.status(401);
                                res.send(response);
                            });

                            // res.status(401).send(response_data);
                        }
                    } catch (error) {
                        response_data = {
                            code: 0,
                            message: t("rest_keyword_invalid_token"),
                        };

                        middleware.encryption(response_data, (response) => {
                            res.status(401);
                            res.send(response);
                        });

                        // res.status(401).send(response_data);
                    }
                } catch (error) {
                    response_data = {
                        code: 0,
                        message: t("rest_keyword_invalid_token"),
                    };

                    // * This function is used to encrypt the response data

                    middleware.encryption(response_data, (response) => {
                        res.status(401);
                        res.send(response);
                    });

                    // res.status(401).send(response_data);
                }
            } else {
                response_data = {
                    code: 0,
                    message: t("token_not_provided"),
                };

                // * This function is used to encrypt the response data

                middleware.encryption(response_data, (response) => {
                    res.status(401);
                    res.send(response);
                });

                // res.status(200).send(response_data);
            }
        } else {
            callback();
        }
    },

    // ! For checking the validation of the token to get user id
    validateDriverToken: async function (req, res, callback) {
        const headertoken = req.headers["token"] != undefined && req.headers["token"] != "" ? req.headers["token"] : "";

        const path = req.path.split("/");

        if (bypassDriverMethods.indexOf(path[1]) === -1) {
            if (headertoken != "") {
                try {
                    const token = cryptLib.decrypt(headertoken, shaKey, process.env.IV);

                    // ! If you want to use jwtToken to create token.
                    // const decode = jwtToken.verify(result[0].token, process.env.KEY);

                    try {
                        const query = `SELECT * FROM driver.tbl_driver_device  WHERE token = '${token}' AND is_active = true AND is_delete = false; `;

                        const result = await conn.query(query, []);

                        if (result.rows.length > 0) {
                            if (req.body === undefined) {
                                req.body = {};
                                req.body.driver_id = result.rows[0].driver_id;
                                callback();
                            } else {
                                req.body.driver_id = result.rows[0].driver_id;
                                callback();
                            }
                        } else {
                            response_data = {
                                code: 0,
                                message: t("rest_keyword_invalid_token"),
                            };

                            middleware.encryption(response_data, (response) => {
                                res.status(401);
                                res.send(response);
                            });

                            // res.status(401).send(response_data);
                        }
                    } catch (error) {
                        response_data = {
                            code: 0,
                            message: t("rest_keyword_invalid_token"),
                        };

                        middleware.encryption(response_data, (response) => {
                            res.status(401);
                            res.send(response);
                        });

                        // res.status(401).send(response_data);
                    }
                } catch (error) {
                    response_data = {
                        code: 0,
                        message: t("rest_keyword_invalid_token"),
                    };

                    // * This function is used to encrypt the response data

                    middleware.encryption(response_data, (response) => {
                        res.status(401);
                        res.send(response);
                    });

                    // res.status(401).send(response_data);
                }
            } else {
                response_data = {
                    code: 0,
                    message: t("token_not_provided"),
                };

                // * This function is used to encrypt the response data

                middleware.encryption(response_data, (response) => {
                    res.status(401);
                    res.send(response);
                });

                // res.status(200).send(response_data);
            }
        } else {
            callback();
        }
    },

    // ! For checking the language in header to give response in that particular language
    extractHeaderLanguage: function (req, res, callback) {
        const headerlang = req.headers["accept-language"] != undefined && req.headers["accept-language"] != "" ? req.headers["accept-language"] : "en";

        if (headerlang == "gu") {
            req.language = gu;
        } else {
            req.language = en;
        }

        localizify.add("en", en).add("gu", gu).setLocale(headerlang);

        callback();
    },

    // ! For encrypting the data using cryptLib
    // decryption: function (encryped_text, callback) {
    //     if (encryped_text != undefined && Object.keys(encryped_text).length !== 0) {
    //         try {
    //             req.body = JSON.parse(cryptLib.decrypt(req.body, shaKey, process.env.IV));
    //             callback(request);
    //         } catch (error) {
    //             callback();
    //         }
    //     } else {
    //         callback();
    //     }
    // },

    // ! For encrypt the response data
    encryption: function (response_data, callback) {
        const response = cryptLib.encrypt(JSON.stringify(response_data), shaKey, process.env.IV);
        callback(response);
    },

    // ! For decoding the encypted request body
    decryption: function (req, res, callback) {
        // req.body != undefined && Object.keys(req.body).length !== 0
        if (!lodash.isEmpty(req.body) && typeof req.body != undefined) {
            try {
                req.body = JSON.parse(cryptLib.decrypt(req.body, shaKey, process.env.IV));
                callback();
            } catch (error) {
                callback();
            }
        } else {
            callback();
        }
    },
};

module.exports = middleware;
