const common = require("../../../../config/common");
const cryptLib = require("cryptlib");
const GLOBALS = require("../../../../config/constant");
const code = require("../../../../config/code");
// const asyncLoop = require("node-async-loop");
const otpGenerator = require("otp-generator");
// const { result } = require("lodash");
// const template = require("../../../config/template");
const jwtToken = require("jsonwebtoken");
const { password } = require("../../../../language/en");
const { logout } = require("../../api_document/route");

// ? It is used to generate a secret key which will be used to encrypt and decrypt the data using cryptLib
const shaKey = cryptLib.getHashSha256(GLOBALS.KEY, 32);

// ? It is used for s3 bucket to store the images
// const AWS = require("aws-sdk");
// const s3 = new AWS.S3();

var Auth = {
    // * This function is used to do the first step of sign - up
    signUpVerification: async (requestData, callback) => {
        try {
            // requestData.password = md5(requestData.password);
            requestData.otp = otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false });

            const uniqueCustomerEmail = await common.uniqueEmail("customer.tbl_customer", requestData.email);
            const uniqueDriverEmail = await common.uniqueEmail("driver.tbl_driver", requestData.email);

            if (uniqueDriverEmail) {
                if (uniqueCustomerEmail) {
                    const uniqueCustomerMobileCountryCode = await common.uniqueMobileCountryCode(requestData.mobile_number, requestData.country_code, "customer.tbl_customer");

                    const uniqueDriverMobileCountryCode = await common.uniqueMobileCountryCode(requestData.mobile_number, requestData.country_code, "driver.tbl_driver");

                    if (uniqueDriverMobileCountryCode) {
                        if (uniqueCustomerMobileCountryCode) {
                            const userData = {
                                first_name: requestData.first_name,
                                last_name: requestData.last_name,
                                email: requestData.email,
                                mobile_number: requestData.mobile_number,
                                country_code: requestData.country_code,
                                otp: requestData.otp,
                                password: cryptLib.encrypt(requestData.password, shaKey, GLOBALS.IV),
                            };

                            const customerId = await common.singleInsert("customer.tbl_customer", userData);

                            if (customerId > 0) {
                                const locationData = {
                                    customer_id: customerId,
                                    latitude,
                                };

                                let updatedId = await common.singleInsert("customer.tbl_location_history");

                                const userDeviceDetails = {
                                    customer_id: customerId,
                                    token: otpGenerator.generate(6, { lowerCaseAlphabets: true, upperCaseAlphabets: true, specialChars: false }),
                                    device_type: requestData.device_type,
                                    device_token: requestData.device_token,
                                    os_version: requestData.os_version,
                                    device_name: requestData.device_name,
                                    model_name: requestData.model_name,
                                    ip: requestData.ip,
                                    uuid: requestData.uuid,
                                };

                                updatedId = await common.singleInsert("customer.tbl_customer_device", userDeviceDetails);

                                const customerData = await common.getCustomerModel(`id = ${customerId}`);

                                if (updatedId > 0) {
                                    return callback(code.SUCCESS, { keyword: "signup_successful", content: "" }, customerData);
                                } else {
                                    return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                                }
                            } else {
                                return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                            }
                        } else {
                            return callback(code.ERROR, { keyword: "rest_keywords_unique_mobile_error", content: "" }, []);
                        }
                    } else {
                        return callback(code.ERROR, { keyword: "rest_keyword_driver_mobile_number_used", content: "" }, []);
                    }
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_unique_email_error", content: "" }, []);
                }
            } else {
                return callback(code.ERROR, { keyword: "rest_keyword_driver_email_used", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used for otp verification api
    requestOtp: async (requestData, callback) => {
        let customerData = await common.getCustomerModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' `);

        if (customerData != null && customerData != undefined) {
            const updatedId = await common.singleUpdate(
                "customer.tbl_customer",
                { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
                `id = ${customerData.customer_details.id}`
            );

            if (updatedId > 0) {
                customerData = await common.getCustomerModel(`id = ${customerData.customer_details.id}`);

                if (customerData != null && customerData != undefined) {
                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, customerData);
                } else {
                    return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, {});
                }
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
            }
        } else {
            return callback(code.DATA_NOT_FOUND, { keyword: "user_not_found", content: "" }, []);
        }
    },

    // * This function is used for otp verification api
    otpVerification: async (requestData, callback) => {
        try {
            let customerData = await common.getCustomerModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' AND otp = ${requestData.otp} `);

            if (customerData != undefined && customerData != null) {
                let updatedId;

                if (customerData.customer_details.profile_steps == "0" || customerData.customer_details.is_verified == false) {
                    updatedId = await common.singleUpdate(
                        "customer.tbl_customer",
                        { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }), steps: "1", is_verified: true },
                        `id = ${customerData.customer_details.id}`
                    );
                } else {
                    updatedId = await common.singleUpdate(
                        "customer.tbl_customer",
                        { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
                        `id = ${customerData.customer_details.id}`
                    );
                }

                if (updatedId > 0) {
                    customerData = await common.getCustomerModel(`id = ${customerData.customer_details.id} `);

                    return callback(code.SUCCESS, { keyword: "otp_verified", content: "" }, customerData);
                } else {
                    return callback(code.ERROR, { keyword: "otp_not_verified", content: "" }, []);
                }
            } else {
                return callback(code.ERROR, { keyword: "wrong_otp", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used for checking mobile whether it exist or not
    mobileVerification: async (requestData, callback) => {
        try {
            const customerData = await common.getCustomerModel(`mobile_number = ${requestData.mobile_number} AND country_code = ${requestData.country_code}`);

            if (customerData != null && customerData != undefined) {
                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, customerData);
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "user_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used to change the password that was forgotten
    changeForgotPassword: async (requestData, callback) => {
        try {
            requestData.new_password = await cryptLib.encrypt(requestData.new_password, shaKey, GLOBALS.IV);

            const result = await common.singleUpdate("customer.tbl_customer", { password: requestData.new_password }, `id = ${requestData.customer_id}`);

            if (result > 0) {
                const customerData = await common.getCustomerModel(`id = ${requestData.customer_id}`);

                return callback(code.SUCCESS, { keyword: "new_password_set", content: "" }, customerData);
            } else {
                return callback(code.ERROR, { keyword: "password_not_changed", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used to change the password
    changePassword: async (requestData, callback) => {
        requestData.old_password = cryptLib.encrypt(requestData.old_password, shaKey, GLOBALS.IV);
        requestData.new_password = cryptLib.encrypt(requestData.new_password, shaKey, GLOBALS.IV);

        try {
            const result = await common.singleUpdate("customer.tbl_customer", { password: requestData.old_password }, `id = ${requestData.customer_id}`);

            if (result > 0) {
                const updatedId = await common.singleUpdate("customer.tbl_customer", { password: requestData.new_password }, `id = ${requestData.customer_id}`);

                if (updatedId > 0) {
                    const customerData = await common.getCustomerModel(`id = ${requestData.customer_id}'`);

                    return callback(code.SUCCESS, { keyword: "new_password_set", content: "" }, customerData);
                } else {
                    return callback(code.ERROR, { keyword: "password_not_changed", content: "" }, []);
                }
            } else {
                return callback(code.ERROR, { keyword: "password_incorrect", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used for login
    login: async (requestData, callback) => {
        try {
            let customerData = await common.getCustomerModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}'  `);

            if (customerData != null) {
                if (customerData.customer_details.steps == "0" || customerData.customer_details.is_verified == false) {
                    return callback(code.OTP_NOT_VERIFIED, { keyword: "password_incorrect", content: "" }, []);
                } else {
                    requestData.password = cryptLib.encrypt(requestData.password, shaKey, GLOBALS.IV);

                    customerData = await common.getCustomerModel(
                        `mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' AND password = '${requestData.password}' `
                    );

                    if (customerData != null) {
                        let userDeviceDetails = {
                            // token: jwtToken.sign({ algorithm: "RS256" }, process.env.KEY, { expiresIn: "7d" }),
                            token: otpGenerator.generate(10, { upperCaseAlphabets: true, specialChars: false, lowerCaseAlphabets: true }),
                            device_type: requestData.device_type,
                            device_token: requestData.device_token,
                            os_version: requestData.os_version,
                            device_name: requestData.device_name,
                            model_name: requestData.model_name,
                            ip: requestData.ip,
                            uuid: requestData.uuid,
                        };

                        const customer_id = await common.singleUpdate("customer.tbl_customer_device", userDeviceDetails, `customer_id = '${customerData.customer_details.id}'`);

                        if (customer_id > 0) {
                            customerData = await common.getCustomerModel(`id = ${customerData.customer_details.id}`);
                            return callback(code.SUCCESS, { keyword: "signup_successful", content: "" }, customerData);
                        } else {
                            return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
                        }
                    } else {
                        return callback(code.ERROR, { keyword: "password_incorrect", content: "" }, []);
                    }
                }
            } else {
                return callback(code.ERROR, { keyword: "data_not_exist", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used to update customer data
    editCustomerData: async (requestData, callback) => {
        try {
            if (requestData.email != undefined) {
                const uniqueCustomerEmail = await common.uniqueEmail("customer.tbl_customer", requestData.email);
                const uniqueDriverEmail = await common.uniqueEmail("driver.tbl_driver", requestData.email);

                if (!uniqueDriverEmail && !uniqueCustomerEmail) {
                    return callback(code.ERROR, { keyword: "rest_keywords_unique_email_error", content: "" }, []);
                }
            }

            if (requestData.mobile_number != undefined && requestData.country_code != undefined) {
                const uniqueCustomerMobileCountryCode = await common.uniqueMobileCountryCode(requestData.mobile_number, requestData.country_code, "customer.tbl_customer");

                const uniqueDriverMobileCountryCode = await common.uniqueMobileCountryCode(requestData.mobile_number, requestData.country_code, "driver.tbl_driver");

                if (!uniqueCustomerMobileCountryCode && !uniqueDriverMobileCountryCode) {
                    return callback(code.ERROR, { keyword: "rest_keywords_unique_email_error", content: "" }, []);
                }

                const updatedId = await common.singleUpdate(
                    "customer.tbl_customer",
                    { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
                    `id = ${customerData.customer_details.id}`
                );
            }

            if (requestData.date_of_birth != null && requestData.date_of_birth != undefined) {
                const result = common.dobIsValid(requestData.date_of_birth);

                if (!result) {
                    return callback(code.ERROR, { keyword: "date_invalid", content: "" }, []);
                }
            }

            const editData = {};

            if (Object.keys(requestData).length > 1) {
                Object.keys(requestData).forEach((key) => {
                    if (key != "customer_id") {
                        editData[key] = requestData[key];
                    }
                });

                const updatedId = await common.singleUpdate("customer.tbl_customer", editData, `id  = ${requestData.customer_id}`);

                if (updatedId > 0) {
                    const customerData = await common.getCustomerModel(`id = ${requestData.customer_id}`);

                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, customerData);
                } else {
                    return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                }
            } else {
                const customerData = await common.getCustomerModel(`id = ${requestData.customer_id}`);

                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, customerData);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
        }
    },

    // * This function is used to update customer data WITH S3
    // editCustomerData: async (requestData, callback) => {
    //     try {
    //         if (requestData.email !== undefined) {
    //             const uniqueCustomerEmail = await common.uniqueEmail("customer.tbl_customer", requestData.email);
    //             const uniqueDriverEmail = await common.uniqueEmail("driver.tbl_driver", requestData.email);

    //             if (!uniqueDriverEmail && !uniqueCustomerEmail) {
    //                 return callback(code.ERROR, { keyword: "rest_keywords_unique_email_error", content: "" }, []);
    //             }
    //         }

    //         if (requestData.mobile_number !== undefined && requestData.country_code !== undefined) {
    //             const uniqueCustomerMobileCountryCode = await common.uniqueMobileCountryCode(requestData.mobile_number, requestData.country_code, "customer.tbl_customer");
    //             const uniqueDriverMobileCountryCode = await common.uniqueMobileCountryCode(requestData.mobile_number, requestData.country_code, "driver.tbl_driver");

    //             if (!uniqueCustomerMobileCountryCode && !uniqueDriverMobileCountryCode) {
    //                 return callback(code.ERROR, { keyword: "rest_keywords_unique_mobile_error", content: "" }, []);
    //             }

    //             await common.singleUpdate(
    //                 "customer.tbl_customer",
    //                 { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
    //                 `id = ${requestData.customer_id}`
    //             );
    //         }

    //         if (requestData.date_of_birth != null && requestData.date_of_birth !== undefined) {
    //             const result = common.dobIsValid(requestData.date_of_birth);

    //             if (!result) {
    //                 return callback(code.ERROR, { keyword: "date_invalid", content: "" }, []);
    //             }
    //         }

    //         const editData = {};

    //         // Handle profile image upload
    //         if (requestData.profile_image) {
    //             const buffer = Buffer.from(requestData.profile_image, "base64");
    //             const s3Params = {
    //                 Bucket: process.env.S3_BUCKET_NAME,
    //                 Key: `profiles/${Date.now()}_${requestData.customer_id}.jpg`, // Generate a unique key
    //                 Body: buffer,
    //                 ContentType: "image/jpeg",
    //                 ACL: "public-read", // Make the file publicly readable
    //             };

    //             const s3Upload = await s3.upload(s3Params).promise();
    //             editData.profile_image = s3Upload.Location; // Store the S3 link
    //         }

    //         if (Object.keys(requestData).length > 1) {
    //             Object.keys(requestData).forEach((key) => {
    //                 if (key !== "customer_id" && key !== "profile_image") {
    //                     editData[key] = requestData[key];
    //                 }
    //             });

    //             const updatedId = await common.singleUpdate("customer.tbl_customer", editData, `id = ${requestData.customer_id}`);

    //             if (updatedId > 0) {
    //                 const customerData = await common.getCustomerModel(`id = ${requestData.customer_id}`);

    //                 return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, customerData);
    //             } else {
    //                 return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
    //             }
    //         } else {
    //             const customerData = await common.getCustomerModel(`id = ${requestData.customer_id}`);

    //             return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, customerData);
    //         }
    //     } catch (error) {
    //         console.error(error); // Log the error for debugging
    //         return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
    //     }
    // },

    // * This function is used to get customer data with device details also
    customerData: async (requestData, callback) => {
        try {
            const userInfo = await common.getCustomerModel(`id = ${requestData.customer_id}`);
            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, userInfo);
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
        }
    },

    // * This function is used to delete the account from the database
    deleteAccount: async (requestData, callback) => {
        try {
            const deviceUpdatedId = await common.singleUpdate("customer.tbl_customer_device", { is_active: false, is_delete: true }, `customer_id = ${requestData.customer_id}`);

            if (deviceUpdatedId != null && deviceUpdatedId) {
                const updatedId = await common.singleUpdate("customer.tbl_customer", { is_active: false, is_delete: true }, `id = ${requestData.customer_id}`);

                if (updatedId != null && updatedId > 0) {
                    return callback(code.SUCCESS, { keyword: "account_deleted", content: "" });
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
                }
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
        }
    },

    // * This function is used to empty token when customer logout
    logout: async (requestData, callback) => {
        try {
            const updatedId = await common.singleUpdate("customer.tbl_customer_device", { token: null }, `is_active = true AND is_delete = false AND customer_id =  ${requestData.customer_id}`);

            if (updatedId != null && updatedId > 0) {
                return callback(code.ERROR, { keyword: "logout", content: "" });
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
        }
    },
};

module.exports = Auth;
