const common = require("../../../../config/common");
const cryptLib = require("cryptlib");
const GLOBALS = require("../../../../config/constant");
const code = require("../../../../config/code");
// const asyncLoop = require("node-async-loop");
const otpGenerator = require("otp-generator");
// const { result } = require("lodash");
// const template = require("../../../config/template");
const jwtToken = require("jsonwebtoken");

// ? It is used to generate a secret key which will be used to encrypt and decrypt the data using cryptLib
const shaKey = cryptLib.getHashSha256(GLOBALS.KEY, 32);

var Auth = {
    // * This function is used to do tinput driver details.
    driverDetails: async (requestData, callback) => {
        try {
            console.log(requestData);

            let updatedId = await common.singleUpdate(
                "driver.tbl_driver",
                {
                    profile_image: requestData.profile_image,
                    delivery_location: requestData.delivery_location,
                    delivery_latitude: requestData.delivery_latitude,
                    delivery_longitude: requestData.delivery_longitude,
                    notification_range: parseInt(requestData.notification_range),
                },
                `id = ${requestData.driver_id} AND is_active = true AND is_delete = false`
            );

            if (updatedId > 0) {
                updatedId = await common.singleUpdate("driver.tbl_driver", { steps: "1" }, ` id = ${requestData.driver_id} AND is_active = true AND is_delete = false`);

                requestData.driver_license.forEach(async (element) => {
                    const insertId = await common.singleInsert("driver.tbl_driver_licence", { driver_id: requestData.driver_id, path: element });
                });

                const vehicleDetailsId = await common.singleInsert("driver.tbl_vehicle", {
                    number_plate: requestData.number_plate,
                    vehicle_type: requestData.vehicle_type,
                    driver_id: requestData.driver_id,
                });

                if (vehicleDetailsId > 0) {
                    // Insert vehicle images
                    for (const element of requestData.vehicle_image) {
                        vehicleImageId = await common.singleInsert("driver.tbl_vehicle_image", { vehicle_id: vehicleDetailsId, path: element });
                    }

                    // Insert registration numbers
                    for (const element of requestData.registration_number) {
                        registrationNumberId = await common.singleInsert("driver.tbl_registration_number", { vehicle_id: vehicleDetailsId, path: element });
                    }

                    // Insert vehicle insurance
                    if (requestData.vehicle_insurance != undefined) {
                        for (const element of requestData.vehicle_insurance) {
                            vehicleInsuranceId = await common.singleInsert("driver.tbl_vehicle_insurance", { vehicle_id: vehicleDetailsId, path: element });
                        }
                    }

                    if (vehicleImageId > 0 && registrationNumberId > 0 && vehicleInsuranceId > 0) {
                        updatedId = await common.singleUpdate("driver.tbl_driver", { steps: "2" }, `id = ${requestData.driver_id} AND is_active = true AND is_delete = false`);

                        if (updatedId > 0) {
                            const driverModel = await common.getDriverModel(`id = ${requestData.driver_id}`);
                            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, driverModel);
                        } else {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        }
                    } else {
                        return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                    }
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                }
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
            }
        } catch (error) {
            console.log(error);

            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used for otp verification api
    requestOtp: async (requestData, callback) => {
        let driverData = await common.getDriverModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' `);

        if (driverData != null && driverData != undefined) {
            const updatedId = await common.singleUpdate(
                "driver.tbl_driver",
                { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
                `id = ${driverData.driver_details.id}`
            );

            if (updatedId > 0) {
                driverData = await common.getDriverModel(`id = ${driverData.driver_details.id}`);

                if (driverData != null && driverData != undefined) {
                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, driverData);
                } else {
                    return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, {});
                }
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
            }
        } else {
            return callback(code.DATA_NOT_FOUND, { keyword: "user_not_found", content: "" }, []);
        }
    },

    // * This function is used for otp verification api
    otpVerification: async (requestData, callback) => {
        try {
            let driverData = await common.getDriverModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' AND otp = ${requestData.otp} `);

            if (driverData != undefined && driverData != null) {
                let updatedId;

                if (driverData.driver_details.steps == "0" || driverData.driver_details.is_verified == false) {
                    updatedId = await common.singleUpdate(
                        "driver.tbl_driver",
                        { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }), steps: "2", is_verified: true },
                        `id = ${driverData.driver_details.id}`
                    );
                } else {
                    updatedId = await common.singleUpdate(
                        "driver.tbl_driver",
                        { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
                        `id = ${driverData.driver_details.id}`
                    );
                }

                if (updatedId > 0) {
                    driverData = await common.getDriverModel(`id = ${driverData.driver_details.id} `);

                    return callback(code.SUCCESS, { keyword: "otp_verified", content: "" }, driverData);
                } else {
                    return callback(code.ERROR, { keyword: "otp_not_verified", content: "" }, []);
                }
            } else {
                return callback(code.ERROR, { keyword: "wrong_otp", content: "" }, []);
            }
        } catch (error) {
            console.log(error);

            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used for login
    login: async (requestData, callback) => {
        try {
            let driverData = await common.getDriverModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' AND id_number = ${requestData.id_number}`);

            if (driverData != null && driverData != undefined) {
                let userDeviceDetails = {
                    // token: jwtToken.sign({ algorithm: "RS256" }, process.env.KEY, { expiresIn: "7d" }),
                    token: otpGenerator.generate(10, { upperCaseAlphabets: true, specialChars: false, lowerCaseAlphabets: true }),
                    device_type: requestData.device_type,
                    device_token: requestData.device_token,
                    os_version: requestData.os_version,
                    device_name: requestData.device_name,
                    model_name: requestData.model_name,
                    ip: requestData.ip,
                    uuid: requestData.uuid,
                };

                const driver_id = await common.singleUpdate("driver.tbl_driver_device", userDeviceDetails, `driver_id = '${driverData.driver_details.id}'`);

                if (driver_id > 0) {
                    driverData = await common.getDriverModel(`id = ${driverData.driver_details.id}`);
                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, driverData);
                } else {
                    return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
                }
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "user_not_found", content: "" }, []);
            }
        } catch (error) {
            console.log(error);

            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used for signing up
    signUp: async (requestData, callback) => {
        try {
            let updatedId;

            let driverData = await common.getDriverModel(`mobile_number = ${requestData.mobile_number} AND country_code = '${requestData.country_code}' AND id_number = ${requestData.id_number}`);

            if (driverData != null && driverData != undefined) {
                const userDeviceDetails = {
                    // token: jwtToken.sign({ algorithm: "RS256" }, process.env.KEY, { expiresIn: "7d" }),
                    token: otpGenerator.generate(10, { upperCaseAlphabets: true, specialChars: false, lowerCaseAlphabets: true }),
                    driver_id: driverData.driver_details.id,
                    device_type: requestData.device_type,
                    device_token: requestData.device_token,
                    os_version: requestData.os_version,
                    device_name: requestData.device_name,
                    model_name: requestData.model_name,
                    ip: requestData.ip,
                    uuid: requestData.uuid,
                };

                const driverDeviceData = await common.getCommonSingleRecord("driver.tbl_driver_device", `driver_id = ${driverData.driver_details.id}`);

                if (driverDeviceData != null && driverDeviceData != undefined) {
                    updatedId = await common.singleUpdate("driver.tbl_driver_device", userDeviceDetails, `driver_id = '${driverData.driver_details.id}'`);
                } else {
                    updatedId = await common.singleInsert("driver.tbl_driver_device", userDeviceDetails);
                }

                if (updatedId > 0) {
                    updatedId = await common.singleUpdate(
                        "driver.tbl_driver",
                        { otp: otpGenerator.generate(4, { upperCaseAlphabets: false, specialChars: false, lowerCaseAlphabets: false }) },
                        `id = ${driverData.driver_details.id}`
                    );

                    if (updatedId > 0) {
                        driverData = await common.getDriverModel(`id = ${driverData.driver_details.id}`);

                        if (driverData != null && driverData != undefined) {
                            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, driverData);
                        } else {
                            return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, {});
                        }
                    } else {
                        return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                    }
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                }
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "user_not_found", content: "" }, []);
            }
        } catch (error) {
            console.log(error);

            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, {});
        }
    },

    // * This function is used to get driver data with device details also
    driverData: async (requestData, callback) => {
        try {
            const userInfo = await common.getDriverModel(`id = ${requestData.driver_id}`);
            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, userInfo);
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
        }
    },
};

module.exports = Auth;
