const express = require("express");
const router = express.Router();
const auth_module = require("./auth_modules");
const common = require("../../../../config/common");
const { t } = require("localizify");
const middleware = require("../../../../middleware/validation");
const { profile_image } = require("../../../../config/constant");

// * for signup step 1
router.post("/sign-up", (req, res) => {
    const rules = {
        mobile_number: "required",
        country_code: "required",
        id_number: "required",
        device_type: "required|in:A,I",
        device_token: "required",
        os_version: "required",
        device_name: "required",
        model_name: "required",
        ip: "required",
        uuid: "required",
    };

    const message = { required: t("required") };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        auth_module.signUp(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * for signup complete profile
router.post("/sign-up-complete-profile", (req, res) => {
    const rules = {
        profile_image: "required",
        driver_license: "required",
        number_plate: "required",
        vehicle_type: "required",
        vehicle_image: "required",
        registration_number: "required",
        vehicle_insurance: "required",
        delivery_location: "required",
        delivery_latitude: "required",
        work_type: "required",
        notification_range: "required",
        delivery_longitude: "required",
    };

    const message = { required: t("required") };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        auth_module.driverDetails(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * For Sending OTP to user OR Updating OTP or Resending OTP
router.post("/request-otp", (req, res) => {
    const rules = {
        country_code: "required",
        mobile_number: "required|numeric",
    };

    const message = { required: t("required") };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        auth_module.requestOtp(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * For OTP Verification
router.post("/otp-verification", (req, res) => {
    const rules = {
        country_code: "required|numeric",
        mobile_number: "required|numeric",
        otp: "required|digits:4",
    };

    const message = { required: t("required"), digits: t("valid") };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        auth_module.otpVerification(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * For login through mobile number , country code and password
router.post("/login", function (req, res) {
    const rules = {
        mobile_number: "required|numeric",
        id_number: "required",
        country_code: "required",
        device_type: "required|in:A,I",
        device_token: "required",
        os_version: "required",
        device_name: "required",
        model_name: "required",
        ip: "required",
        uuid: "required",
    };
    const message = { required: req.language.required, numerid: t("valid") };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        auth_module.login(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * To get data of the particular customer
router.post("/driver-data", function (req, res) {
    auth_module.driverData(req.body, function (code, message, data) {
        common.send_response(req.body, res, code, message, data);
    });
});

module.exports = router;
