const common = require("../../../../config/common");
const conn = require("../../../../config/database");
// const constant = require("../../../../config/constant");
// const md5 = require("md5");
const code = require("../../../../config/code");
const asyncLoop = require("node-async-loop");
const { forEach } = require("lodash");
// const { t } = require("localizify");
// const otpGenerator = require("otp-generator");
// const { email } = require("../../../../language/en");
// const { result } = require("lodash");
// const template = require("../../../../config/template");
// const stripe = require("../../../../config/stripe");

let home = {
    // * This function is used for categories display
    categoryDisplay: async (requestData, callback) => {
        try {
            const result = await common.getCommonMultipleRecord("customer.tbl_categories", `parent_id IS NULL AND is_active=true AND is_delete=false`);

            if (result != null && result != undefined) {
                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, result);
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used for restaurant display on home page
    restaurantListing: (requestData, callback) => {
        conn.query(
            `SELECT r.*,
                ROUND((6371 * acos(
                    cos(radians(${requestData.customer_latitude})) * cos(radians(r.latitude::float)) * 
                    cos(radians(r.longitude::float) - radians(${requestData.customer_longitude})) + 
                    sin(radians(${requestData.customer_latitude})) * sin(radians(r.latitude::float))
                ))::numeric, 2) AS distance,
                COALESCE((
                    SELECT CASE WHEN w.id IS NOT NULL THEN 1 END 
                    FROM customer.tbl_wishlist AS w 
                    WHERE w.is_active = true AND w.customer_id = ${requestData.customer_id} AND w.restaurant_id = r.id
                ), 0) AS is_like
            FROM 
                customer.tbl_restaurant AS r
            WHERE r.is_active = true AND r.is_delete = false
             ORDER BY distance;`,
            [],
            (error, result) => {
                if (error) {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, null);
                } else {
                    if (result.rows.length > 0) {
                        asyncLoop(
                            result.rows,
                            async (item, next) => {
                                if (item.is_offer_available === true) {
                                    const offer = await common.getCommonSingleRecord("customer.tbl_offer", `restaurant_id = ${item.id} `);

                                    if (offer != null && offer != undefined) {
                                        item.offer_details = offer;
                                        next();
                                    } else {
                                        item.offer_details = [];
                                        next(error);
                                    }
                                } else {
                                    next();
                                }
                            },
                            (error) => {
                                if (error) {
                                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                                } else {
                                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, result.rows);
                                }
                            }
                        );
                    } else {
                        return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
                    }
                }
            }
        );
    },

    // * This function is used to inser location history searched by customer
    insertLocationHistory: async (requestData, callback) => {
        try {
            let locationHistoryData = await common.getCommonMultipleRecord(
                "customer.tbl_location_history",
                `customer_id = ${requestData.customer_id} AND latitude = '${requestData.latitude}' AND longitude = '${requestData.longitude}' AND is_active = true AND is_delete = false`
            );

            if (locationHistoryData == null) {
                const customerLocationData = {
                    latitude: requestData.latitude,
                    longitude: requestData.longitude,
                    customer_id: requestData.customer_id,
                    location: requestData.location,
                };

                const insertId = await common.singleInsert("customer.tbl_location_history", customerLocationData);

                if (insertId > 0) {
                    locationHistoryData = await common.getCommonMultipleRecord("customer.tbl_location_history", `customer_id = ${requestData.customer_id} AND is_active = true AND is_delete = false`);

                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, locationHistoryData);
                } else {
                    return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
                }
            } else {
                locationHistoryData = await common.getCommonMultipleRecord("customer.tbl_location_history", `customer_id = ${requestData.customer_id} AND is_active = true AND is_delete = false`);

                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, locationHistoryData);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to get location history of particular user
    locationHistory: async (requestData, callback) => {
        try {
            const result = await common.getCommonMultipleRecord("customer.tbl_location_history", `customer_id = ${requestData.customer_id} AND is_active = true AND is_delete = false`);

            if (result != null) {
                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, result);
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to get location history of particular user
    searchFilterData: async (requestData, callback) => {
        try {
            let dishConditions = ``;
            let restaurantConditions = `is_active = true AND is_delete = false `;

            let data = {};

            if (requestData.searchText != null && requestData.searchText != undefined) {
                restaurantConditions += `OR name like '%${requestData.searchText}%' `;
                dishConditions += dishConditions === `` ? `is_active = true AND is_delete = false OR name like '%${requestData.searchText}%' ` : `OR name like '%${requestData.searchText}%' `;
            }

            if (requestData.ratings != null && requestData.ratings != undefined) {
                dishConditions += dishConditions === `` ? `is_active = true AND is_delete = false OR average_rating >= ${requestData.ratings} ` : `OR average_rating >= ${requestData.ratings} `;

                dishConditions += `OR average_rating >= ${requestData.ratings} `;
                restaurantConditions += `OR average_rating >= ${requestData.ratings} `;
            }

            if (requestData.is_offer != null && requestData.is_offer != undefined) {
                const offer = await common.getCommonMultipleRecord("customer.tbl_offer", "is_active = true and is_delete = false");

                let dishId = ``;
                let restaurantId = ``;
                let restaurantIdArray = [];

                offer.forEach((element) => {
                    if (element.restaurant_id != null && restaurantIdArray.indexOf(element.restaurant_id) == -1) {
                        restaurantIdArray.push(element.restaurant_id);
                        restaurantId += element.restaurant_id + ",";
                    }
                });

                restaurantConditions += `OR id = ANY(string_to_array(trim(trailing ',' from '${restaurantId}'), ',')::int[]) `;
            }

            if (requestData.max_price != null && requestData.max_price != undefined && requestData.min_price != null && requestData.min_price != undefined) {
                dishConditions +=
                    dishConditions === ``
                        ? `is_active = true AND is_delete = false OR price between ${requestData.min_price} AND ${requestData.max_price} `
                        : `OR price between ${requestData.min_price} AND ${requestData.max_price} `;
            }

            if (requestData.category_id != null && requestData.category_id != undefined) {
                let categoryId = "";

                requestData.category_id.forEach((element) => {
                    categoryId += element + ",";
                });

                dishConditions +=
                    dishConditions === ``
                        ? `is_active = true AND is_delete = false OR id = ANY(string_to_array(trim(trailing ',' from '${categoryId}'), ',')::int[])`
                        : `OR id = ANY(string_to_array(trim(trailing ',' from '${categoryId}'), ',')::int[])`;

                dishConditions += `OR id = ANY(string_to_array(trim(trailing ',' from '${categoryId}'), ',')::int[])`;

                const query = `SELECT CONCAT(c.restaurant_id) FROM customer.tbl_combination AS c WHERE c.category_id = ANY(string_to_array(trim(trailing ',' from '${categoryId}'), ',')::int[]) GROUP BY c.restaurant_id`;

                const restaurantData = await conn.query(query, []);

                let restaurantId = ``;

                if (restaurantData.rows.length > 0) {
                    restaurantData.rows.forEach((element) => {
                        restaurantId += element.concat + ",";
                    });

                    restaurantConditions += `OR id = ANY(string_to_array(trim(trailing ',' from '${restaurantId}'), ',')::int[])`;
                }
            }

            if (requestData.location_range != null && requestData.location_range != undefined) {
                restaurantConditions += `OR ROUND((6371 * acos(
                            cos(radians(${requestData.customer_latitude})) * cos(radians(r.latitude::numeric)) * 
                            cos(radians(r.longitude::numeric) - radians(${requestData.customer_longitude})) + 
                            sin(radians(${requestData.customer_latitude})) * sin(radians(r.latitude::numeric))
                        ))::numeric, 2) < ${requestData.location_range}`;
            }

            if (restaurantConditions == ``) {
                const query = `SELECT r.*,
                        ROUND((6371 * acos(
                            cos(radians(${requestData.customer_latitude})) * cos(radians(r.latitude::numeric)) * 
                            cos(radians(r.longitude::numeric) - radians(${requestData.customer_longitude})) + 
                            sin(radians(${requestData.customer_latitude})) * sin(radians(r.latitude::numeric))
                        ))::numeric, 2) AS distance,
                        COALESCE((  
                            SELECT CASE WHEN w.id IS NOT NULL THEN 1 END 
                            FROM customer.tbl_wishlist AS w 
                            WHERE w.is_active = true AND w.customer_id = ${requestData.customer_id} AND w.restaurant_id = r.id
                        ), 0) AS is_like
                    FROM 
                        customer.tbl_restaurant AS r 
                    WHERE 
                        r.is_active = true AND r.is_delete = false
                    ORDER BY distance;
                    `;

                const result = await conn.query(query, []);

                if (result.rows.length > 0) {
                    data.restaunrants = result.rows;
                } else {
                    data.restaunrants = [];
                }
            } else {
                const query = `SELECT r.*,
                        ROUND((6371 * acos(
                            cos(radians(${requestData.customer_latitude})) * cos(radians(r.latitude::float)) * 
                            cos(radians(r.longitude::float) - radians(${requestData.customer_longitude})) + 
                            sin(radians(${requestData.customer_latitude})) * sin(radians(r.latitude::float))
                        ))::numeric, 2) AS distance,
                        COALESCE((
                            SELECT CASE WHEN w.id IS NOT NULL THEN 1 END 
                            FROM customer.tbl_wishlist AS w 
                            WHERE w.is_active = true AND w.customer_id = ${requestData.customer_id} AND w.restaurant_id = r.id
                        ), 0) AS is_like
                    FROM 
                        customer.tbl_restaurant AS r WHERE ${restaurantConditions}
                    ORDER BY distance;
                    `;

                const result = await conn.query(query, []);

                if (result.rows.length > 0) {
                    data.restaunrants = result.rows;
                } else {
                    data.restaunrants = [];
                }
            }

            if (dishConditions == ``) {
                data.dishes = [];
            } else {
                const dishData = await common.getCommonMultipleRecord("customer.tbl_dishes", dishConditions);

                if (dishData != null) {
                    data.dishes = dishData;
                } else {
                    data.dishes = [];
                }
            }

            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, data);
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to get restaurant details
    restaurantDetails: async (requestData, callback) => {
        let data = {};

        try {
            const restaurantDetails = await common.getCommonSingleRecord("customer.tbl_restaurant", `id = ${requestData.restaurant_id} AND is_active = true AND is_delete = false`);

            if (restaurantDetails != null) {
                data.restaurant_details = restaurantDetails;

                if (data.restaurant_details.is_offer_available == true) {
                    const offers = await common.getCommonMultipleRecord("customer.tbl_offer", `restaurant_id =  ${data.restaurant_details.id} AND is_active = true AND is_delete = false`);

                    if (offers != null && offers != undefined) {
                        data.restaurant_details.offers = offers;
                    } else {
                        data.restaurant_details.offers = [];
                    }
                }
            }

            const categoriesId = await common.getCommonMultipleRecord("customer.tbl_combination", `restaurant_id = ${restaurantDetails.id} AND is_active = true AND is_delete = false`);

            let category_id = "";

            if (categoriesId != null) {
                categoriesId.forEach((element) => {
                    category_id += element.category_id + ",";
                });

                const categoriesData = await common.getCommonMultipleRecord(
                    "customer.tbl_categories",
                    `id = ANY(string_to_array(trim(trailing ',' from '${category_id}'), ',')::int[]) AND is_active = true AND is_delete = false`
                );

                if (categoriesData != null) {
                    data.categories_details = categoriesData;
                } else {
                    data.categories_details = [];
                }
            } else {
                data.restaurant_categories = [];
            }

            if (data.categories_details.length > 0) {
                data.categories_details.forEach(async (element) => {
                    const dishesDetails = await common.getCommonMultipleRecord(
                        "customer.tbl_dishes",
                        `category_id = ${element.id} AND restaurant_id = ${requestData.restaurant_id} AND is_active = true AND is_delete = false`
                    );

                    if (dishesDetails != null && dishesDetails.length > 0) {
                        element.dishes = dishesDetails;

                        asyncLoop(
                            element.dishes,
                            async (item, next) => {
                                if (item.is_offer_available == true) {
                                    const offers = await common.getCommonMultipleRecord("customer.tbl_offer", `is_active = true AND is_delete = false  AND dish_id = ${item.id}`);

                                    if (offers != null && offers != undefined) {
                                        item.offers = offers;
                                        next();
                                    } else {
                                        item.offers = [];
                                        next();
                                    }
                                } else {
                                    next();
                                }
                            },
                            (error) => {
                                if (error) {
                                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                                } else {
                                    return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, data);
                                }
                            }
                        );
                    } else {
                        element.dishes = [];
                    }
                });
            } else {
                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, data);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to get particular dish details
    particularDishDetails: async (requestData, callback) => {
        try {
            const result = await common.getCommonSingleRecord("customer.tbl_dishes", `id = ${requestData.dish_id} AND is_active = true AND is_delete = false`);

            if (result != null) {
                if (result.is_offer_available == true) {
                    const offers = await common.getCommonMultipleRecord("customer.tbl_offer", `is_active = true AND  is_delete = false  AND dish_id = ${requestData.dish_id}`);

                    if (offers != null && offers != undefined) {
                        result.offers = offers;
                    } else {
                        result.offers = [];
                    }
                }

                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, result);
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to get particular category dishes list with particular restaurant
    categorywiseParticularRestaurantDishDetails: async (requestData, callback) => {
        try {
            let categoryId = ``;

            requestData.category_id.forEach((element) => {
                categoryId += element + ",";
            });

            const dishData = await common.getCommonMultipleRecord(
                "customer.tbl_dishes",
                `bbcategory_id = ANY(string_to_array(trim(trailing ',' from '${categoryId}'), ',')::int[]) AND restaurant_id = ${requestData.restaurant_id} AND is_active = true AND is_delete = false`
            );

            if (dishData != null) {
                asyncLoop(
                    dishData,
                    async (item, next) => {
                        if (item.is_offer_available == true) {
                            const offer = await common.getCommonMultipleRecord("customer.tbl_offer", `is_active = true AND is_delete = false AND dish_id = ${item.id}`);

                            if (offer != null && offer != undefined) {
                                item.offers = offer;
                                next();
                            } else {
                                item.offers = [];
                                next();
                            }
                        } else {
                            next();
                        }
                    },
                    (error) => {
                        if (error) {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        } else {
                            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, dishData);
                        }
                    }
                );
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to show restaurant list based on particular categoriess
    restaurantCategorywise: async (requestData, callback) => {
        let data = {};

        try {
            let categoryId = "";

            requestData.category_id.forEach((element) => {
                categoryId += element + ",";
            });

            let query = `SELECT CONCAT(c.restaurant_id) FROM customer.tbl_combination AS c WHERE c.is_active = true AND c.is_delete = false AND c.category_id = ANY(string_to_array(trim(trailing ',' from '${categoryId}'), ',')::int[]) GROUP BY c.restaurant_id`;

            const restaurantData = await conn.query(query, []);

            let restaurantId = ``;

            if (restaurantData.rows.length > 0) {
                restaurantData.rows.forEach((element) => {
                    restaurantId += element.concat + ",";
                });

                const result = await common.getCommonMultipleRecord(
                    "customer.tbl_restaurant",
                    `id = ANY(string_to_array(trim(trailing ',' from '${restaurantId}'), ',')::int[]) AND is_active = true AND is_delete = false`
                );

                if (result != null) {
                    asyncLoop(
                        result,
                        async (item, next) => {
                            if (item.is_offer_available == true) {
                                const offers = await common.getCommonMultipleRecord("customer.tbl_offer", `restaurant_id = ${item.id} AND is_active = true AND is_delete = false`);

                                if (offers != null && offers != undefined) {
                                    item.offers = offers;
                                    next();
                                } else {
                                    item.offers = [];
                                    next();
                                }
                            } else {
                                next();
                            }
                        },
                        (error) => {
                            if (error) {
                                callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" });
                            } else {
                                callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, result);
                            }
                        }
                    );
                } else {
                    return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
                }
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to show restaurant details and dishes based on particular categories
    categorywiseDishDetails: async (requestData, callback) => {
        let data = {};

        try {
            const restaurantDetails = await common.getCommonSingleRecord("customer.tbl_restaurant", `id = ${requestData.restaurant_id}`);

            if (restaurantDetails != null) {
                data.restaurant_details = restaurantDetails;
            } else {
                data.restaurant_details = [];
            }

            let categoriesId = ``;

            requestData.category_id.forEach((element) => {
                categoriesId += element + ",";
            });

            const dishData = await common.getCommonMultipleRecord(
                "customer.tbl_dishes",
                `category_id = ANY(string_to_array(trim(trailing ',' from '${categoriesId}'), ',')::int[]) AND restaurant_id = ${requestData.restaurant_id} AND is_active = true AND is_delete = false`
            );

            if (dishData != null) {
                data.dish_details = dishData;
            } else {
                data.dish_details = [];
            }

            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, data);
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to insert review rating of the particular restaurant
    insertRestaurantRating: async (requestData, callback) => {
        try {
            const restaurantRatingData = await common.getCommonSingleRecord(
                "customer.tbl_restaurant_rating",
                `is_active = true AND is_delete = false AND  customer_id = ${requestData.customer_id} AND restaurant_id = ${requestData.restaurant_id}`
            );

            if (restaurantRatingData != null && restaurantRatingData.rating != undefined) {
                const updatedId = await common.singleUpdate(
                    "customer.tbl_restaurant_rating",
                    { rating: parseFloat(requestData.rating).toFixed(1), review: requestData.review },
                    ` id = ${restaurantRatingData.id}`
                );

                if (updatedId > 0) {
                    return callback(code.SUCCESS, { keyword: "data_added", content: "" }, []);
                } else {
                    return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                }
            } else {
                const ratingData = {
                    restaurant_id: requestData.restaurant_id,
                    rating: parseFloat(requestData.rating).toFixed(1),
                    review: requestData.review,
                    customer_id: requestData.customer_id,
                };

                const ratingId = await common.singleInsert("customer.tbl_restaurant_rating", ratingData);

                if (ratingId != null) {
                    return callback(code.SUCCESS, { keyword: "data_added", content: "" }, []);
                } else {
                    return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                }
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to insert review rating of the particular dish
    insertDishRating: async (requestData, callback) => {
        try {
            const dishRatingData = await common.getCommonSingleRecord(
                "customer.tbl_dish_rating",
                `is_active = true AND is_delete = false AND  customer_id = ${requestData.customer_id} AND dish_id = ${requestData.dish_id}`
            );

            if (dishRatingData != null && dishRatingData.rating != undefined) {
                const updatedId = await common.singleUpdate(
                    "customer.tbl_dish_rating",
                    { rating: parseFloat(requestData.rating).toFixed(1), review: requestData.review },
                    ` id = ${dishRatingData.id}`
                );

                if (updatedId > 0) {
                    return callback(code.SUCCESS, { keyword: "data_added", content: "" }, []);
                } else {
                    return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                }
            } else {
                const ratingData = {
                    dish_id: requestData.dish_id,
                    rating: parseFloat(requestData.rating).toFixed(1),
                    review: requestData.review,
                    customer_id: requestData.customer_id,
                };

                const ratingId = await common.singleInsert("customer.tbl_dish_rating", ratingData);

                if (ratingId != null) {
                    return callback(code.SUCCESS, { keyword: "data_added", content: "" }, []);
                } else {
                    return callback(code.ERROR, { keyword: "data_not_submitted", content: "" }, []);
                }
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to list review rating of the particular dish
    listDishRating: async (requestData, callback) => {
        try {
            const dishRatingData = await common.getCommonMultipleRecord("customer.tbl_dish_rating", `is_active = true AND is_delete = false AND dish_id = ${requestData.dish_id}`);

            if (dishRatingData != null && dishRatingData != undefined) {
                asyncLoop(
                    dishRatingData,
                    async (item, next) => {
                        let customerData = await common.getCommonSingleRecord("customer.tbl_customer", `is_active = true AND is_delete = false AND id = ${item.customer_id}`);

                        if (customerData != null && customerData != undefined) {
                            item.customer_data = customerData;
                            next();
                        } else {
                            next();
                        }
                    },
                    (error) => {
                        if (error) {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        } else {
                            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, dishRatingData);
                        }
                    }
                );
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to list review rating of the particular dish
    listRestaurantRating: async (requestData, callback) => {
        try {
            const restaurantRatingData = await common.getCommonMultipleRecord(
                "customer.tbl_restaurant_rating",
                `is_active = true AND is_delete = false AND restaurant_id = ${requestData.restaurant_id}`
            );

            if (restaurantRatingData != null && restaurantRatingData != undefined) {
                asyncLoop(
                    restaurantRatingData,
                    async (item, next) => {
                        let customerData = await common.getCommonSingleRecord("customer.tbl_customer", `is_active = true AND is_delete = false AND id = ${item.customer_id}`);

                        if (customerData != null && customerData != undefined) {
                            item.customer_data = customerData;
                            next();
                        } else {
                            next();
                        }
                    },
                    (error) => {
                        if (error) {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        } else {
                            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, restaurantRatingData);
                        }
                    }
                );
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to insert contact us data of particular user
    contactUs: async (requestData, callback) => {
        try {
            const contactUsData = {
                customer_id: requestData.customer_id,
                full_name: requestData.full_name,
                email: requestData.email,
                subject: requestData.subject,
                description: requestData.description,
            };

            const contactUsId = await common.singleInsert("customer.tbl_contact_us", contactUsData);

            if (contactUsId != null && contactUsId > 0) {
                asyncLoop(
                    requestData.media,
                    async (item, next) => {
                        const imageData = {
                            contact_us_id: contactUsId,
                            path: item,
                        };

                        const insertId = await common.singleInsert("customer.tbl_contact_us_media", imageData);

                        if (insertId != null && insertId > 0) {
                            next();
                        } else {
                            next(error);
                        }
                    },
                    async (error) => {
                        if (error) {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        } else {
                            return callback(code.SUCCESS, { keyword: "data_added", content: "" }, []);
                        }
                    }
                );
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to insert report data of particular user
    report: async (requestData, callback) => {
        try {
            const reportData = {
                customer_id: requestData.customer_id,
                description: requestData.description,
                reason: requestData.reason,
            };

            const reportId = await common.singleInsert("customer.tbl_report", reportData);

            if (reportId != null && reportId > 0) {
                asyncLoop(
                    requestData.media,
                    async (item, next) => {
                        const imageData = {
                            report_id: reportId,
                            path: item,
                        };

                        const insertId = await common.singleInsert("customer.tbl_report_media", imageData);

                        if (insertId != null && insertId > 0) {
                            next();
                        } else {
                            next(error);
                        }
                    },
                    async (error) => {
                        if (error) {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        } else {
                            return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" });
                        }
                    }
                );
            } else {
                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to insert into cart data and cart details ( Pending )
    insertCartDetails: async (requestData, callback) => {
        try {
            const cartData = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete = false AND customer_id = ${requestData.customer_id}`);

            if (cartData == null && cartData == undefined) {
                let dishData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete = false AND  id = ${requestData.product_details[0].product_id}`);

                const chargesData = await common.getCommonSingleRecord("customer.tbl_charges", `is_active =  true AND is_delete = false AND restaurant_id  = ${dishData.restaurant_id}`);

                const requestCartData = {
                    customer_id: requestData.customer_id,
                    discount_price: 0,
                    delivery_charges: 10,
                    service_fee: chargesData.service_fee,
                    late_night_charges: 0,
                    rainy_charges: 0,
                    is_coupon_applied: false,
                    coupon_code: null,
                };

                const cartId = await common.singleInsert("customer.tbl_cart", requestCartData);

                if (cartId > 0) {
                    asyncLoop(
                        requestData.product_details,
                        async (item, next) => {
                            let dishData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete = false AND  id = ${item.product_id}`);

                            const cartDetailsData = {
                                cart_id: cartId,
                                product_id: dishData.id,
                                per_product_price: dishData.price,
                                quantity: item.quantity,
                                total: parseFloat(dishData.price * item.quantity).toFixed(2),
                            };

                            const insertId = await common.insertCartDetails(cartDetailsData);

                            if (insertId != null) {
                                next();
                            } else {
                                next(error);
                            }
                        },
                        async (error) => {
                            if (error) {
                                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                            } else {
                                let data = {};

                                data.cart_data = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete =false AND id = ${cartId}`);

                                data.products = await common.getCommonMultipleRecord("customer.tbl_cart_details", `is_active = true AND is_delete = false AND cart_id = ${cartId}`);

                                asyncLoop(
                                    data.products,
                                    async (item, next) => {
                                        const producData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete=false AND id = ${item.product_id}`);

                                        if (producData != null && producData != undefined) {
                                            item.product_details = producData;
                                            next();
                                        } else {
                                            next();
                                        }
                                    },
                                    async (error) => {
                                        if (error) {
                                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                                        } else {
                                            return callback(code.SUCCESS, { keyword: "cart_success", content: "" }, data);
                                        }
                                    }
                                );
                            }
                        }
                    );
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                }
            } else {
                const cartDetails = await common.getCommonSingleRecord("customer.tbl_cart_details", `is_active = true AND is_delete = false AND  cart_id = ${cartData.id}`);

                for (let element of requestData.product_details) {
                    const productDetails = await common.getCommonSingleRecord(`customer.tbl_dishes`, `is_active = true AND is_delete = false AND id = ${element.product_id}`);

                    const cartDetailsData = {
                        cart_id: cartData.id,
                        product_id: element.product_id,
                        per_product_price: productDetails.price,
                        quantity: element.quantity,
                        total: parseFloat(productDetails.price).toFixed(2) * parseInt(element.quantity),
                    };

                    if (cartDetails == null && cartDetails == undefined) {
                        const insertId = await common.insertCartDetails(cartDetailsData);
                    } else {
                        const cartProductRestaurantId = await common.getCommonSingleRecord(`customer.tbl_dishes`, `is_active = true AND is_delete = false AND id = ${cartDetails.product_id}`);

                        if (productDetails.restaurant_id === cartProductRestaurantId.restaurant_id) {
                            const insertId = await common.insertCartDetails(cartDetailsData);
                        } else {
                            return callback(code.ERROR, { keyword: "different_product", content: "" });
                        }
                    }
                }

                let data = {};

                data.cart_data = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete =false AND id = ${cartData.id}`);

                data.products = await common.getCommonMultipleRecord("customer.tbl_cart_details", `is_active = true AND is_delete = false AND cart_id = ${cartData.id}`);

                asyncLoop(
                    data.products,
                    async (item, next) => {
                        const producData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete=false AND id = ${item.product_id}`);

                        if (producData != null && producData != undefined) {
                            item.product_details = producData;
                            next();
                        } else {
                            next();
                        }
                    },
                    async (error) => {
                        if (error) {
                            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                        } else {
                            return callback(code.SUCCESS, { keyword: "cart_success", content: "" }, data);
                        }
                    }
                );
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to update the product quantity of the particular product
    updateQuantity: async (requestData, callback) => {
        try {
            const cartData = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete = false AND customer_id = ${requestData.customer_id}`);

            const productData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete = false AND id = ${requestData.product_id}`);

            if (cartData != null) {
                const updatedId = await common.singleUpdate(
                    "customer.tbl_cart_details",
                    { quantity: requestData.quantity, total: parseFloat(productData.price).toFixed(2) * parseInt(requestData.quantity) },
                    `is_active = true AND is_delete = false AND cart_id = ${cartData.id} AND product_id = ${requestData.product_id}`
                );

                if (updatedId != null && updatedId > 0) {
                    let data = {};

                    data.cart_data = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete =false AND id = ${cartData.id}`);

                    data.products = await common.getCommonMultipleRecord("customer.tbl_cart_details", `is_active = true AND is_delete = false AND cart_id = ${cartData.id}`);

                    asyncLoop(
                        data.products,
                        async (item, next) => {
                            const producData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete=false AND id = ${item.product_id}`);

                            if (producData != null && producData != undefined) {
                                item.product_details = producData;
                                next();
                            } else {
                                next();
                            }
                        },
                        async (error) => {
                            if (error) {
                                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                            } else {
                                return callback(code.SUCCESS, { keyword: "cart_success", content: "" }, data);
                            }
                        }
                    );
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                }
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "rest_keywords_cart_not_found", content: "" });
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to remove product from cart
    removeProductFromCart: async (requestData, callback) => {
        try {
            const cartData = await common.getCommonSingleRecord("customer.tbl_cart", ` is_active = true AND is_delete = false `);

            const cartDetailsData = await common.getCommonMultipleRecord(
                "customer.tbl_cart_details",
                `is_active = true AND is_delete = false AND cart_id = ${cartData.id}  AND product_id = ${requestData.product_id}`
            );

            if (cartDetailsData != null && cartDetailsData != undefined) {
                const updateId = await common.singleDelete(
                    "customer.tbl_cart_details",
                    `is_active = true AND  is_delete = false AND cart_id = ${cartData.id} AND  product_id = ${requestData.product_id}`
                );

                if (updateId != null && updateId > 0) {
                    let data = {};

                    data.cart_data = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete =false AND id = ${cartData.id}`);

                    data.products = await common.getCommonMultipleRecord("customer.tbl_cart_details", `is_active = true AND is_delete = false AND cart_id = ${cartData.id}`);

                    asyncLoop(
                        data.products,
                        async (item, next) => {
                            const producData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete=false AND id = ${item.product_id}`);

                            if (producData != null && producData != undefined) {
                                item.product_details = producData;
                                next();
                            } else {
                                next();
                            }
                        },
                        async (error) => {
                            if (error) {
                                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                            } else {
                                return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, data);
                            }
                        }
                    );
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                }
            } else {
                return callback(code.ERROR, { keyword: "rest_keyword_product_not_found_in_cart", content: "" }, []);
            }
        } catch (error) {
            console.log(error);

            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to insert the order data
    insertOrder: async (requestData, callback) => {
        try {
            const cartData = await common.getCommonSingleRecord("customer.tbl_cart", `is_active = true AND is_delete=false AND customer_id = ${requestData.customer_id}`);

            if (cartData != null && cartData != undefined) {
                const requestOrderData = {
                    customer_id: requestData.customer_id,
                    discount_price: cartData.discount_price,
                    delivery_charges: cartData.delivery_charges,
                    service_fee: cartData.service_fee,
                    late_night_charges: cartData.late_night_charges,
                    rainy_charges: cartData.rainy_charges,
                    is_coupon_applied: cartData.is_coupon_applied,
                    coupon_code: cartData.coupon_code,
                    location_id: requestData.location_id,
                    total_amount: cartData.total_amount,
                    total_quantity: cartData.total_quantity,
                    discount_price: cartData.discount_price,
                };

                const orderId = await common.singleInsert("customer.tbl_order", requestOrderData);

                if (orderId != null && orderId > 0) {
                    const cartDetailsData = await common.getCommonMultipleRecord("customer.tbl_cart_details", `is_active = true AND is_delete = false AND  cart_id = ${cartData.id}`);

                    asyncLoop(
                        cartDetailsData,
                        async (item, next) => {
                            const orderDetailsData = {
                                order_id: orderId,
                                product_id: item.product_id,
                                per_product_price: item.per_product_price,
                                quantity: item.quantity,
                                total: item.total,
                            };

                            const orderDetailsId = await common.singleInsert("customer.tbl_order_details", orderDetailsData);

                            if (orderDetailsId != null && orderDetailsId > 0) {
                                next();
                            } else {
                                next(error);
                            }
                        },
                        async (error) => {
                            if (error) {
                                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, error);
                            } else {
                                const deleteCart = await common.singleDelete("customer.tbl_cart", ` id = ${cartData.id} `);

                                if (deleteCart != null && deleteCart > 0) {
                                    let data = {};

                                    data.order_data = await common.getCommonSingleRecord("customer.tbl_order", `is_active = true AND is_delete =false AND id = ${orderId}`);

                                    data.products = await common.getCommonMultipleRecord("customer.tbl_order_details", `is_active = true AND is_delete = false AND order_id = ${orderId}`);

                                    data.location_data = await common.getCommonSingleRecord(
                                        "customer.tbl_location_history",
                                        `is_active = true AND is_delete = false AND id = ${requestData.location_id}`
                                    );

                                    asyncLoop(
                                        data.products,
                                        async (item, next) => {
                                            const productData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete=false AND id = ${item.product_id}`);

                                            if (productData != null && productData != undefined) {
                                                item.product_details = productData;
                                                next();
                                            } else {
                                                next();
                                            }
                                        },
                                        async (error) => {
                                            if (error) {
                                                return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                                            } else {
                                                data.restaurant_details = await common.getCommonSingleRecord(
                                                    "customer.tbl_restaurant",
                                                    `is_active = true AND is_delete = false AND id = ${data.products[0].product_details.restaurant_id}`
                                                );

                                                return callback(code.SUCCESS, { keyword: "order_success", content: "" }, data);
                                            }
                                        }
                                    );
                                } else {
                                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                                }
                            }
                        }
                    );
                } else {
                    return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                }
            } else {
                return callback(code.DATA_NOT_FOUND, { keyword: "data_not_found", content: "" }, []);
            }
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },

    // * This function is used to get orde details
    getOrderDetails: async (requestData, callback) => {
        try {
            let data = {};

            data.order_data = await common.getCommonSingleRecord("customer.tbl_order", `is_active = true AND is_delete =false AND customer_id = ${requestData.customer_id}`);

            data.products = await common.getCommonMultipleRecord("customer.tbl_order_details", `is_active = true AND is_delete = false AND order_id = ${data.order_data.id}`);

            data.location_data = await common.getCommonSingleRecord("customer.tbl_location_history", `is_active = true AND is_delete = false AND id = ${data.order_data.location_id}`);

            asyncLoop(
                data.products,
                async (item, next) => {
                    const productData = await common.getCommonSingleRecord("customer.tbl_dishes", `is_active = true AND is_delete=false AND id = ${item.product_id}`);

                    if (productData != null && productData != undefined) {
                        item.product_details = productData;
                        next();
                    } else {
                        next();
                    }
                },
                async (error) => {
                    if (error) {
                        return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
                    } else {
                        data.restaurant_details = await common.getCommonSingleRecord(
                            "customer.tbl_restaurant",
                            `is_active = true AND is_delete = false AND id = ${data.products[0].product_details.restaurant_id}`
                        );

                        return callback(code.SUCCESS, { keyword: "rest_keywords_success", content: "" }, data);
                    }
                }
            );
        } catch (error) {
            return callback(code.ERROR, { keyword: "rest_keywords_something_went_wrong", content: "" }, []);
        }
    },
};

module.exports = home;
