const express = require("express");
const router = express.Router();
const home_module = require("./home_modules");
const common = require("../../../../config/common");
const middleware = require("../../../../middleware/validation");
const { t } = require("localizify");

// * This route is used for categories listing
router.post("/categories-listing", function (req, res) {
    home_module.categoryDisplay(req, function (code, message, data) {
        common.send_response(req, res, code, message, data);
    });
});

// * This route is used for restaurant listing
router.post("/restaurnat-listing", function (req, res) {
    const rules = {
        customer_latitude: "required",
        customer_longitude: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.restaurantListing(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This route is used to insert location history search by particular customer
router.post("/insert-location-history", function (req, res) {
    const rules = {
        location: "required",
        latitude: "required",
        longitude: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.insertLocationHistory(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This route is used for location history for particular user
router.post("/location-history", function (req, res) {
    home_module.locationHistory(req.body, function (code, message, data) {
        common.send_response(req.body, res, code, message, data);
    });
});

// * This route is used for search and filteration
router.post("/search-filter", function (req, res) {
    const rules = {
        customer_latitude: "required",
        customer_longitude: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.searchFilterData(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to shows the restaurant details
router.post("/restaurant-details", function (req, res) {
    const rules = {
        restaurant_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.restaurantDetails(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to shows the restaurant details
router.post("/particular-restaurant-dishes", function (req, res) {
    const rules = {
        restaurant_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.restaurantDetails(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to shows the restaurant's particular dish details
router.post("/dish-details", function (req, res) {
    const rules = {
        dish_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.particularDishDetails(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to shows the dish's ratings details
router.post("/dish-ratings", function (req, res) {
    const rules = {
        dish_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.listDishRating(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to shows the restaurant's ratings details
router.post("/restaurant-ratings", function (req, res) {
    const rules = {
        restaurant_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.listRestaurantRating(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to show restaurant list based on the categories
router.post("/categorywise-restaurant-listings", function (req, res) {
    const rules = {
        category_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.restaurantCategorywise(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

//* The route is used to show restaurant list based on the categories
router.post("/categorywise-dish-details", function (req, res) {
    const rules = {
        category_id: "required",
        restaurant_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.categorywiseParticularRestaurantDishDetails(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to insert rating of a particular restaurant
router.post("/insert-update-restaurant-rating", function (req, res) {
    const rules = {
        rating: "required|numeric",
        restaurant_id: "required",
        review: "required",
    };

    const message = { required: req.language.required, numeric: req.language.valid };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.insertRestaurantRating(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to insert rating of a particular dish
router.post("/insert-update-dish-rating", function (req, res) {
    const rules = {
        rating: "required|numeric",
        dish_id: "required",
        review: "required",
    };

    const message = { required: req.language.required, numeric: req.language.valid };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.insertDishRating(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to insert contact us data
router.post("/insert-contact-us", function (req, res) {
    const rules = {
        full_name: "required",
        email: "required",
        subject: "required",
        description: "required",
        media: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.contactUs(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to insert report data
router.post("/insert-report", function (req, res) {
    const rules = {
        reason: "required",
        description: "required",
        media: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.report(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to insert cart details
router.post("/insert-cart", function (req, res) {
    const rules = {
        product_details: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.insertCartDetails(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to update quantity in the cart
router.post("/update-quantity", function (req, res) {
    const rules = {
        product_id: "required",
        quantity: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.updateQuantity(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to insert order details
router.post("/insert-order", function (req, res) {
    const rules = {
        location_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.insertOrder(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

// * This function is used to get order details of particular user
router.post("/order-details", function (req, res) {
    home_module.getOrderDetails(req.body, function (code, message, data) {
        common.send_response(req.body, res, code, message, data);
    });
});

// * This function is used to remove particular product from the cart
router.post("/remove-product-cart", function (req, res) {
    const rules = {
        product_id: "required",
    };

    const message = { required: req.language.required };

    if (middleware.checkValidationRules(req.body, res, rules, message)) {
        home_module.removeProductFromCart(req.body, function (code, message, data) {
            common.send_response(req.body, res, code, message, data);
        });
    }
});

module.exports = router;
